module Camera (Projection (..),
               mkPersp,
               Camera (..),
               projMat, camMat
              )
       where
import LinPrim

data Projection a = Ortho {
                    width :: a,
                    height :: a,
                    near :: a,
                    far :: a
                    }
                  | Persp {
                    hFov :: a,
                    vFov :: a,
                    nearFrust :: a,
                    farFrust :: a
                    }
                  deriving (Eq, Show)

vFovFromH :: Floating a => Int -> Int -> a -> a
vFovFromH w h hFov = 2*atan(tan (hFov/2) * r)
  where r = (fromIntegral h) / (fromIntegral w)

--helper to make frustrum of correct aspect ratio
mkPersp :: Floating a => Int -> Int -> a -> a -> a -> Projection a
mkPersp w h hFov nFrust fFrust
  = Persp hFov (vFovFromH w h hFov) nFrust fFrust

projMat :: Floating a => Projection a -> Mat a
projMat (Ortho w h n f) = mOrth w h n f
projMat (Persp h v n f) = mProj h v n f

data Camera a = Cam {
  origin :: HVec a,
  dir :: HVec a,
  roll :: a,
  proj :: Projection a
  }
              deriving (Eq, Show)


camMat :: (RealFloat a ,Floating a) => Camera a -> Mat a
camMat (Cam p face rol proj)
  = (projMat proj) `mMul` (mRotRoll rol)`mMul`(mLook face) `mMul` (mTranslation (vNeg p))
