module Colour
       (Colour(..),
        toPixel, fromPixel,
        cSplat, brown,
        black, cornflowerBlue,
        red, green, blue,
        white, sunshine
       )
       where
import Codec.Picture.Types
import Data.Word (Word8, Word16)
import Utils

--Colour type that allows componentwise algebraic manipulation
data Colour = Colour {cR::Float, cG::Float, cB::Float}
              deriving (Eq, Show)
cSplat :: Float -> Colour
cSplat s
  = Colour s s s

toPixel :: Colour -> PixelRGB8
toPixel (Colour r g b)
  = PixelRGB8 (scale r) (scale g) (scale b)
    where
      scale a = round (clip 0 255 (a*255))

class Pixel a => CPixel a where
  fromPixel :: a -> Colour

instance CPixel PixelRGBF where
  fromPixel (PixelRGBF r g b) = Colour r g b
instance CPixel PixelRGB8 where
  fromPixel (PixelRGB8 r g b) = Colour (toNormal r)
                                       (toNormal g)
                                       (toNormal b)
instance CPixel PixelRGB16 where
  fromPixel (PixelRGB16 r g b) = Colour (toNormal r)
                                        (toNormal g)
                                        (toNormal b)
instance CPixel PixelRGBA8 where
  fromPixel (PixelRGBA8 r g b _) = fromPixel (PixelRGB8 r g b)
instance CPixel PixelRGBA16 where
  fromPixel (PixelRGBA16 r g b _) = fromPixel (PixelRGB16 r g b)
instance CPixel Word8 where
  fromPixel y = cSplat (toNormal y)
instance CPixel Word16 where
  fromPixel y = cSplat (toNormal y)
instance CPixel Float where
  fromPixel y = cSplat y
instance CPixel PixelYA8 where
  fromPixel (PixelYA8 y _) = fromPixel y
instance CPixel PixelYA16 where
  fromPixel (PixelYA16 y _) = fromPixel y
----------------------------------------------
instance CPixel PixelYCbCr8 where
  fromPixel (PixelYCbCr8 y cb cr) = undefined
instance CPixel PixelCMYK8 where
  fromPixel (PixelCMYK8 c m y k) = undefined
instance CPixel PixelCMYK16 where
  fromPixel (PixelCMYK16 c m y k) = undefined
----------------------------------------------

instance Num (Colour) where
  (+) = cAdd
  (*) = cMul
  (-) = cSub
  abs = cAbs
  signum = cSignum
  fromInteger = cFromInteger
  negate = cNeg

instance Fractional (Colour) where
  (/) = cDiv
  fromRational = cFromRational


cAdd :: Colour -> Colour -> Colour
cAdd (Colour r g b) (Colour r' g' b')
  = Colour (r+r') (g+g') (b+b')

cSub :: Colour -> Colour -> Colour
cSub (Colour r g b) (Colour r' g' b')
  = Colour (r-r') (g-g') (b-b')

cMul :: Colour -> Colour -> Colour
cMul (Colour r g b) (Colour r' g' b')
  = Colour (r*r') (g*g') (b*b')

cNeg :: Colour -> Colour
cNeg (Colour r g b)
  = Colour (-r) (-g) (-r)

cAbs :: Colour -> Colour
cAbs (Colour r g b)
  = Colour (abs r) (abs g) (abs b)

cSignum :: Colour -> Colour
cSignum (Colour r g b)
  = Colour (signum r) (signum g) (signum b)

cFromInteger :: Integer -> Colour
cFromInteger a = cSplat (fromInteger a)

cDiv :: Colour -> Colour -> Colour
cDiv (Colour r g b) (Colour r' g' b')
  = Colour (r/r') (g/g') (r/r')

cFromRational :: Rational -> Colour
cFromRational a = cSplat (fromRational a)

--Handy colours
black = Colour 0.05 0.05 0.05
white = Colour 0.9 0.9 0.9
red   = Colour 0.9 0.0 0.0
green = Colour 0.0 0.9 0.0
blue  = Colour 0.0 0.0 0.9
brown = Colour 0.3 0.15 0.0
sunshine = Colour 0.9 0.85 0.40
cornflowerBlue = Colour 0.39 0.58 0.93
