module Lights
       (Lights(..),Material(..),
        Scene(..), DirectL(..),
        flatMaterial, mkMaterial
       )
       where
import Codec.Picture
import Codec.Picture.Types (createMutableImage, unsafeFreezeImage)
import LinPrim
import Camera
import Colour

--Ambient light, directedlights
data Lights = Lights Colour [DirectL]
            deriving (Eq, Show)
                     
data Material =
  Material {
    mTexture :: (Image PixelRGB8),
    mShine :: Int,
    mSpec :: Colour,
    mDiff :: Colour,
    mAmb :: Colour
    }
instance Show Material where
  show m = "Material {shine: " ++ show (mShine m) ++ ", spec: " ++
           show (mSpec m) ++ ", diff: " ++ show (mDiff m) ++
           ", amb: " ++ show (mAmb m) ++ "}\n"

flatMaterial :: Colour -> Int -> Colour -> Colour -> Colour -> IO (Material)
flatMaterial bg shine spec diff amb = do
  mTex <- createMutableImage 1 1 (toPixel bg)
  tex <- unsafeFreezeImage mTex
  return (Material tex shine spec diff amb)

mkMaterial :: FilePath -> Int -> Colour -> Colour -> Colour -> IO (Material)
mkMaterial fn shine spec diff amb = do
  tex <- readAsRGB8 fn
  return (Material tex shine spec diff amb)

readAsRGB8 :: FilePath -> IO (Image PixelRGB8)
readAsRGB8 fn = do
  edImg <- readImage fn
  case edImg of
   (Right dImg) ->
     case dImg of
     ImageRGB8   i -> return i
     ImageY8     i -> toRGB8 i
     ImageY16    i -> toRGB8 i
     ImageYF     i -> toRGB8 i
     ImageYA8    i -> toRGB8 i
     ImageRGB16  i -> toRGB8 i
     ImageRGBF   i -> toRGB8 i
     ImageRGBA8  i -> toRGB8 i
     ImageRGBA16 i -> toRGB8 i
     ImageYCbCr8 i -> toRGB8 i
     ImageCMYK8  i -> toRGB8 i
     ImageCMYK16 i -> toRGB8 i
   (Left err) -> fail err
  where
    toRGB8 i = return (pixelMap (toPixel.fromPixel) i)
            
            
data DirectL =
  DirectL {
    dDir :: HVec Float,
    dDiffLum :: Colour,
    dSpecLum :: Colour
    }
  deriving (Eq, Show)


data Scene = Scene {scLights :: Lights, scCam :: (Camera Float)}
           deriving (Eq, Show)



               
