module Utils (
  RastAct,
  toNormal,
  radFromDeg, degFromRad,
  clip, enumBtw,
  lerp, vLerp,
  fileExtension,
  split
  )
       where
import Codec.Picture.Types(PixelRGB8)
import Data.List (isSuffixOf)
import Data.Char (toLower)
import LinPrim

--Rasteriser action, (x,y,depth,colour)
type RastAct = (Int,Int,Float,PixelRGB8)

radFromDeg :: Floating a => a -> a
radFromDeg d = (pi * d) / 180

degFromRad :: Floating a => a -> a
degFromRad r = (r * 180) / pi

toNormal :: (Fractional b, Integral a,Bounded a) => a -> b
toNormal n = fromIntegral n / fromIntegral (maxBound `asTypeOf` n)

lerp :: Num a => a -> a -> a -> a
lerp x x' s = (1-s)*x + s*x'

vLerp :: (Ord a,Num a) => HVec a -> HVec a -> a -> HVec a
vLerp p p' s = ((1-s)`sDot`p) `vAdd` (s`sDot`p') 

clip :: Ord a => a -> a -> a -> a
clip lo hi x = max lo (min hi x)

enumBtw :: (Ord a, Enum a) => a -> a -> [a]
enumBtw a b |  a < b   = [a.. b]
            |  a > b   = [b.. a]
            |otherwise = []

fileExtension :: FilePath -> String -> FilePath
fileExtension ext fn | ext' `isSuffixOf` fn' = fn
                     | otherwise           = fn ++ ext
  where ext' = map toLower ext
        fn' = map toLower fn

split1 :: Char -> String -> (String, Maybe String)
split1 k (c:cs) | c == k  = ([],Just cs)
                |otherwise= (\(a,b)->(c:a,b)) $ split1 k cs
split1 k [] = ([],Nothing)

split :: Char -> String -> [String]
split k = f . split1 k
  where f (a,b) = case b of
                   Nothing -> [a]
                   Just b' -> a : split k b'
